@@ -23,12 +23,10 @@ from matplotlib._pylab_helpers import Gcf
 from matplotlib.figure import Figure
 
 from matplotlib.widgets import SubplotTool
-try:
-    import matplotlib.backends.qt_editor.figureoptions as figureoptions
-except ImportError:
-    figureoptions = None
+import matplotlib.backends.qt_editor.figureoptions as figureoptions
 
-from .qt_compat import QtCore, QtGui, QtWidgets, _getSaveFileName, __version__
+from .qt_compat import (QtCore, QtGui, QtWidgets, _getSaveFileName,
+                        __version__, is_pyqt5)
 from matplotlib.backends.qt_editor.formsubplottool import UiSubplotTool
 
 backend_version = __version__
@@ -132,9 +130,17 @@ def _create_qApp():
         app = QtWidgets.QApplication.instance()
         if app is None:
             # check for DISPLAY env variable on X11 build of Qt
-            if hasattr(QtGui, "QX11Info"):
+            if is_pyqt5():
+                try:
+                    from PyQt5 import QtX11Extras
+                    is_x11_build = True
+                except ImportError:
+                    is_x11_build = False
+            else:
+                is_x11_build = hasattr(QtGui, "QX11Info")
+            if is_x11_build:
                 display = os.environ.get('DISPLAY')
-                if display is None or not re.search(':\d', display):
+                if display is None or not re.search(r':\d', display):
                     raise RuntimeError('Invalid DISPLAY variable')
 
             qApp = QtWidgets.QApplication([str(" ")])
@@ -142,6 +148,13 @@ def _create_qApp():
         else:
             qApp = app
 
+    if is_pyqt5():
+        try:
+            qApp.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
+            qApp.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
+        except AttributeError:
+            pass
+
 
 class Show(ShowBase):
     def mainloop(self):
@@ -150,6 +163,7 @@ class Show(ShowBase):
         global qApp
         qApp.exec_()
 
+
 show = Show()
 
 
@@ -193,16 +207,6 @@ class TimerQT(TimerBase):
         self._timer.timeout.connect(self._on_timer)
         self._timer_set_interval()
 
-    def __del__(self):
-        # Probably not necessary in practice, but is good behavior to
-        # disconnect
-        try:
-            TimerBase.__del__(self)
-            self._timer.timeout.disconnect(self._on_timer)
-        except RuntimeError:
-            # Timer C++ object already deleted
-            pass
-
     def _timer_set_single_shot(self):
         self._timer.setSingleShot(self._single)
 
@@ -234,68 +238,84 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
         # NB: Using super for this call to avoid a TypeError:
         # __init__() takes exactly 2 arguments (1 given) on QWidget
         # PyQt5
+        # The need for this change is documented here
+        # http://pyqt.sourceforge.net/Docs/PyQt5/pyqt4_differences.html#cooperative-multi-inheritance
         super(FigureCanvasQT, self).__init__(figure=figure)
         self.figure = figure
         self.setMouseTracking(True)
-        self._idle = True
-        # hide until we can test and fix
-        # self.startTimer(backend_IdleEvent.milliseconds)
         w, h = self.get_width_height()
         self.resize(w, h)
 
-    def __timerEvent(self, event):
-        # hide until we can test and fix
-        self.mpl_idle_event(event)
+    @property
+    def _dpi_ratio(self):
+        # Not available on Qt4 or some older Qt5.
+        try:
+            return self.devicePixelRatio()
+        except AttributeError:
+            return 1
+
+    def get_width_height(self):
+        w, h = FigureCanvasBase.get_width_height(self)
+        return int(w / self._dpi_ratio), int(h / self._dpi_ratio)
 
     def enterEvent(self, event):
-        FigureCanvasBase.enter_notify_event(self, event)
+        FigureCanvasBase.enter_notify_event(self, guiEvent=event)
 
     def leaveEvent(self, event):
         QtWidgets.QApplication.restoreOverrideCursor()
-        FigureCanvasBase.leave_notify_event(self, event)
+        FigureCanvasBase.leave_notify_event(self, guiEvent=event)
+
+    def mouseEventCoords(self, pos):
+        """Calculate mouse coordinates in physical pixels
+
+        Qt5 use logical pixels, but the figure is scaled to physical
+        pixels for rendering.   Transform to physical pixels so that
+        all of the down-stream transforms work as expected.
+
+        Also, the origin is different and needs to be corrected.
+
+        """
+        dpi_ratio = self._dpi_ratio
+        x = pos.x()
+        # flip y so y=0 is bottom of canvas
+        y = self.figure.bbox.height / dpi_ratio - pos.y()
+        return x * dpi_ratio, y * dpi_ratio
 
     def mousePressEvent(self, event):
-        x = event.pos().x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.pos().y()
+        x, y = self.mouseEventCoords(event.pos())
         button = self.buttond.get(event.button())
         if button is not None:
-            FigureCanvasBase.button_press_event(self, x, y, button)
+            FigureCanvasBase.button_press_event(self, x, y, button,
+                                                guiEvent=event)
         if DEBUG:
             print('button pressed:', event.button())
 
     def mouseDoubleClickEvent(self, event):
-        x = event.pos().x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.pos().y()
+        x, y = self.mouseEventCoords(event.pos())
         button = self.buttond.get(event.button())
         if button is not None:
             FigureCanvasBase.button_press_event(self, x, y,
-                                                button, dblclick=True)
+                                                button, dblclick=True,
+                                                guiEvent=event)
         if DEBUG:
             print('button doubleclicked:', event.button())
 
     def mouseMoveEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
-        FigureCanvasBase.motion_notify_event(self, x, y)
+        x, y = self.mouseEventCoords(event)
+        FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=event)
         # if DEBUG: print('mouse move')
 
     def mouseReleaseEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
+        x, y = self.mouseEventCoords(event)
         button = self.buttond.get(event.button())
         if button is not None:
-            FigureCanvasBase.button_release_event(self, x, y, button)
+            FigureCanvasBase.button_release_event(self, x, y, button,
+                                                  guiEvent=event)
         if DEBUG:
             print('button released')
 
     def wheelEvent(self, event):
-        x = event.x()
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y()
+        x, y = self.mouseEventCoords(event)
         # from QWheelEvent::delta doc
         if event.pixelDelta().x() == 0 and event.pixelDelta().y() == 0:
             steps = event.angleDelta().y() / 120
@@ -303,7 +323,7 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
             steps = event.pixelDelta().y()
 
         if steps != 0:
-            FigureCanvasBase.scroll_event(self, x, y, steps)
+            FigureCanvasBase.scroll_event(self, x, y, steps, guiEvent=event)
             if DEBUG:
                 print('scroll event: delta = %i, '
                       'steps = %i ' % (event.delta(), steps))
@@ -312,7 +332,7 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
         key = self._get_key(event)
         if key is None:
             return
-        FigureCanvasBase.key_press_event(self, key)
+        FigureCanvasBase.key_press_event(self, key, guiEvent=event)
         if DEBUG:
             print('key press', key)
 
@@ -320,23 +340,22 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
         key = self._get_key(event)
         if key is None:
             return
-        FigureCanvasBase.key_release_event(self, key)
+        FigureCanvasBase.key_release_event(self, key, guiEvent=event)
         if DEBUG:
             print('key release', key)
 
     def resizeEvent(self, event):
-        w = event.size().width()
-        h = event.size().height()
+        w = event.size().width() * self._dpi_ratio
+        h = event.size().height() * self._dpi_ratio
         if DEBUG:
             print('resize (%d x %d)' % (w, h))
             print("FigureCanvasQt.resizeEvent(%d, %d)" % (w, h))
         dpival = self.figure.dpi
         winch = w / dpival
         hinch = h / dpival
-        self.figure.set_size_inches(winch, hinch)
+        self.figure.set_size_inches(winch, hinch, forward=False)
         FigureCanvasBase.resize_event(self)
-        self.draw()
-        self.update()
+        self.draw_idle()
         QtWidgets.QWidget.resizeEvent(self, event)
 
     def sizeHint(self):
@@ -417,19 +436,6 @@ class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
 
     stop_event_loop.__doc__ = FigureCanvasBase.stop_event_loop_default.__doc__
 
-    def draw_idle(self):
-        'update drawing area only if idle'
-        d = self._idle
-        self._idle = False
-
-        def idle_draw(*args):
-            try:
-                self.draw()
-            finally:
-                self._idle = True
-        if d:
-            QtCore.QTimer.singleShot(0, idle_draw)
-
 
 class MainWindow(QtWidgets.QMainWindow):
     closing = QtCore.Signal()
@@ -475,10 +481,14 @@ class FigureManagerQT(FigureManagerBase):
 
         self.window._destroying = False
 
+        # add text label to status bar
+        self.statusbar_label = QtWidgets.QLabel()
+        self.window.statusBar().addWidget(self.statusbar_label)
+
         self.toolbar = self._get_toolbar(self.canvas, self.window)
         if self.toolbar is not None:
             self.window.addToolBar(self.toolbar)
-            self.toolbar.message.connect(self._show_message)
+            self.toolbar.message.connect(self.statusbar_label.setText)
             tbs_height = self.toolbar.sizeHint().height()
         else:
             tbs_height = 0
@@ -495,17 +505,14 @@ class FigureManagerQT(FigureManagerBase):
 
         if matplotlib.is_interactive():
             self.window.show()
+            self.canvas.draw_idle()
 
         def notify_axes_change(fig):
             # This will be called whenever the current axes is changed
             if self.toolbar is not None:
                 self.toolbar.update()
         self.canvas.figure.add_axobserver(notify_axes_change)
-
-    @QtCore.Slot()
-    def _show_message(self, s):
-        # Fixes a PySide segfault.
-        self.window.statusBar().showMessage(s)
+        self.window.raise_()
 
     def full_screen_toggle(self):
         if self.window.isFullScreen():
@@ -557,7 +564,7 @@ class FigureManagerQT(FigureManagerBase):
         self.window.close()
 
     def get_window_title(self):
-        return str(self.window.windowTitle())
+        return six.text_type(self.window.windowTitle())
 
     def set_window_title(self, title):
         self.window.setWindowTitle(title)
@@ -578,7 +585,11 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         NavigationToolbar2.__init__(self, canvas)
 
     def _icon(self, name):
-        return QtGui.QIcon(os.path.join(self.basedir, name))
+        if is_pyqt5():
+            name = name.replace('.png', '_large.png')
+        pm = QtGui.QPixmap(os.path.join(self.basedir, name))
+        pm.setDevicePixelRatio(self.canvas._dpi_ratio)
+        return QtGui.QIcon(pm)
 
     def _init_toolbar(self):
         self.basedir = os.path.join(matplotlib.rcParams['datapath'], 'images')
@@ -588,17 +599,16 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
                 self.addSeparator()
             else:
                 a = self.addAction(self._icon(image_file + '.png'),
-                                         text, getattr(self, callback))
+                                   text, getattr(self, callback))
                 self._actions[callback] = a
                 if callback in ['zoom', 'pan']:
                     a.setCheckable(True)
                 if tooltip_text is not None:
                     a.setToolTip(tooltip_text)
-
-        if figureoptions is not None:
-            a = self.addAction(self._icon("qt4_editor_options.png"),
-                               'Customize', self.edit_parameters)
-            a.setToolTip('Edit curves line and axes parameters')
+                if text == 'Subplots':
+                    a = self.addAction(self._icon("qt4_editor_options.png"),
+                                       'Customize', self.edit_parameters)
+                    a.setToolTip('Edit axis, curve and image parameters')
 
         self.buttons = {}
 
@@ -611,46 +621,54 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
                     QtCore.Qt.AlignRight | QtCore.Qt.AlignTop)
             self.locLabel.setSizePolicy(
                 QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding,
-                                  QtWidgets.QSizePolicy.Ignored))
+                                      QtWidgets.QSizePolicy.Ignored))
             labelAction = self.addWidget(self.locLabel)
             labelAction.setVisible(True)
 
         # reference holder for subplots_adjust window
         self.adj_window = None
 
-    if figureoptions is not None:
-        def edit_parameters(self):
-            allaxes = self.canvas.figure.get_axes()
-            if len(allaxes) == 1:
-                axes = allaxes[0]
-            else:
-                titles = []
-                for axes in allaxes:
-                    title = axes.get_title()
-                    ylabel = axes.get_ylabel()
-                    label = axes.get_label()
-                    if title:
-                        fmt = "%(title)s"
-                        if ylabel:
-                            fmt += ": %(ylabel)s"
-                        fmt += " (%(axes_repr)s)"
-                    elif ylabel:
-                        fmt = "%(axes_repr)s (%(ylabel)s)"
-                    elif label:
-                        fmt = "%(axes_repr)s (%(label)s)"
-                    else:
-                        fmt = "%(axes_repr)s"
-                    titles.append(fmt % dict(title=title,
-                                         ylabel=ylabel, label=label,
-                                         axes_repr=repr(axes)))
-                item, ok = QtWidgets.QInputDialog.getItem(
-                    self.parent, 'Customize', 'Select axes:', titles, 0, False)
-                if ok:
-                    axes = allaxes[titles.index(six.text_type(item))]
-                else:
-                    return
+        # Esthetic adjustments - we need to set these explicitly in PyQt5
+        # otherwise the layout looks different - but we don't want to set it if
+        # not using HiDPI icons otherwise they look worse than before.
+        if is_pyqt5():
+            self.setIconSize(QtCore.QSize(24, 24))
+            self.layout().setSpacing(12)
 
-            figureoptions.figure_edit(axes, self)
+    if is_pyqt5():
+        # For some reason, self.setMinimumHeight doesn't seem to carry over to
+        # the actual sizeHint, so override it instead in order to make the
+        # aesthetic adjustments noted above.
+        def sizeHint(self):
+            size = super(NavigationToolbar2QT, self).sizeHint()
+            size.setHeight(max(48, size.height()))
+            return size
+
+    def edit_parameters(self):
+        allaxes = self.canvas.figure.get_axes()
+        if not allaxes:
+            QtWidgets.QMessageBox.warning(
+                self.parent, "Error", "There are no axes to edit.")
+            return
+        if len(allaxes) == 1:
+            axes = allaxes[0]
+        else:
+            titles = []
+            for axes in allaxes:
+                name = (axes.get_title() or
+                        " - ".join(filter(None, [axes.get_xlabel(),
+                                                 axes.get_ylabel()])) or
+                        "<anonymous {} (id: {:#x})>".format(
+                            type(axes).__name__, id(axes)))
+                titles.append(name)
+            item, ok = QtWidgets.QInputDialog.getItem(
+                self.parent, 'Customize', 'Select axes:', titles, 0, False)
+            if ok:
+                axes = allaxes[titles.index(six.text_type(item))]
+            else:
+                return
+
+        figureoptions.figure_edit(axes, self)
 
     def _update_buttons_checked(self):
         # sync button checkstates to match active mode
@@ -666,12 +684,12 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         self._update_buttons_checked()
 
     def dynamic_update(self):
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def set_message(self, s):
         self.message.emit(s)
         if self.coordinates:
-            self.locLabel.setText(s.replace(', ', '\n'))
+            self.locLabel.setText(s)
 
     def set_cursor(self, cursor):
         if DEBUG:
@@ -689,6 +707,9 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
         rect = [int(val)for val in (min(x0, x1), min(y0, y1), w, h)]
         self.canvas.drawRectangle(rect)
 
+    def remove_rubberband(self):
+        self.canvas.drawRectangle(None)
+
     def configure_subplots(self):
         image = os.path.join(matplotlib.rcParams['datapath'],
                              'images', 'matplotlib.png')
@@ -730,7 +751,7 @@ class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
                 self.canvas.print_figure(six.text_type(fname))
             except Exception as e:
                 QtWidgets.QMessageBox.critical(
-                    self, "Error saving file", str(e),
+                    self, "Error saving file", six.text_type(e),
                     QtWidgets.QMessageBox.Ok, QtWidgets.QMessageBox.NoButton)
 
 
@@ -752,13 +773,17 @@ class SubplotToolQt(SubplotTool, UiSubplotTool):
 
         self.defaults = {}
         for attr in ('left', 'bottom', 'right', 'top', 'wspace', 'hspace', ):
-            self.defaults[attr] = getattr(self.targetfig.subplotpars, attr)
+            val = getattr(self.targetfig.subplotpars, attr)
+            self.defaults[attr] = val
             slider = getattr(self, 'slider' + attr)
+            txt = getattr(self, attr + 'value')
             slider.setMinimum(0)
             slider.setMaximum(1000)
             slider.setSingleStep(5)
+            # do this before hooking up the callbacks
+            slider.setSliderPosition(int(val * 1000))
+            txt.setText("%.2f" % val)
             slider.valueChanged.connect(getattr(self, 'func' + attr))
-
         self._setSliderPositions()
 
     def _setSliderPositions(self):
@@ -773,7 +798,7 @@ class SubplotToolQt(SubplotTool, UiSubplotTool):
         self.targetfig.subplots_adjust(left=val)
         self.leftvalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def funcright(self, val):
         if val == self.sliderleft.value():
@@ -782,7 +807,7 @@ class SubplotToolQt(SubplotTool, UiSubplotTool):
         self.targetfig.subplots_adjust(right=val)
         self.rightvalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def funcbottom(self, val):
         if val == self.slidertop.value():
@@ -791,7 +816,7 @@ class SubplotToolQt(SubplotTool, UiSubplotTool):
         self.targetfig.subplots_adjust(bottom=val)
         self.bottomvalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def functop(self, val):
         if val == self.sliderbottom.value():
@@ -800,31 +825,31 @@ class SubplotToolQt(SubplotTool, UiSubplotTool):
         self.targetfig.subplots_adjust(top=val)
         self.topvalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def funcwspace(self, val):
         val /= 1000.
         self.targetfig.subplots_adjust(wspace=val)
         self.wspacevalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def funchspace(self, val):
         val /= 1000.
         self.targetfig.subplots_adjust(hspace=val)
         self.hspacevalue.setText("%.2f" % val)
         if self.drawon:
-            self.targetfig.canvas.draw()
+            self.targetfig.canvas.draw_idle()
 
     def functight(self):
         self.targetfig.tight_layout()
         self._setSliderPositions()
-        self.targetfig.canvas.draw()
+        self.targetfig.canvas.draw_idle()
 
     def reset(self):
         self.targetfig.subplots_adjust(**self.defaults)
         self._setSliderPositions()
-        self.targetfig.canvas.draw()
+        self.targetfig.canvas.draw_idle()
 
 
 def error_msg_qt(msg, parent=None):
@@ -846,7 +871,7 @@ def exception_handler(type, value, tb):
     if hasattr(value, 'strerror') and value.strerror is not None:
         msg += value.strerror
     else:
-        msg += str(value)
+        msg += six.text_type(value)
 
     if len(msg):
         error_msg_qt(msg)
